@@ -44,6 +44,7 @@
 
 #include "mir/events/event_builders.h"
 #include "mir/logging/logger.h"
+#include "mir_error.h"
 
 #include <algorithm>
 #include <cstddef>
@@ -128,6 +129,7 @@ mir::protobuf::SurfaceParameters serialize_spec(MirSurfaceSpec const& spec)
     SERIALIZE_OPTION_IF_SET(width_inc);
     SERIALIZE_OPTION_IF_SET(height_inc);
     SERIALIZE_OPTION_IF_SET(shell_chrome);
+    SERIALIZE_OPTION_IF_SET(confine_pointer);
     // min_aspect is a special case (below)
     // max_aspect is a special case (below)
 
@@ -172,6 +174,22 @@ mir::protobuf::SurfaceParameters serialize_spec(MirSurfaceSpec const& spec)
         }
     }
 
+    if (spec.streams.is_set())
+    {
+        for(auto const& stream : spec.streams.value())
+        {
+            auto const new_stream = message.add_stream();
+            new_stream->set_displacement_x(stream.displacement.dx.as_int());
+            new_stream->set_displacement_y(stream.displacement.dy.as_int());
+            new_stream->mutable_id()->set_value(stream.stream_id);
+            if (stream.size.is_set())
+            {
+                new_stream->set_width(stream.size.value().width.as_int());
+                new_stream->set_height(stream.size.value().height.as_int());
+            }
+        }
+    }
+
     return message;
 }
 
@@ -346,8 +364,6 @@ mf::SurfaceId MirConnection::next_error_id(std::unique_lock<std::mutex> const&)
 void MirConnection::surface_created(SurfaceCreationRequest* request)
 {
     std::unique_lock<decltype(mutex)> lock(mutex);
-    std::shared_ptr<MirSurface> surf {nullptr};
-    std::shared_ptr<mcl::ClientBufferStream> stream {nullptr};
     //make sure this request actually was made.
     auto request_it = std::find_if(surface_requests.begin(), surface_requests.end(),
         [&request](std::shared_ptr<MirConnection::SurfaceCreationRequest> r){ return request == r.get(); });
@@ -358,29 +374,34 @@ void MirConnection::surface_created(SurfaceCreationRequest* request)
     auto callback = request->cb;
     auto context = request->context;
     auto const& spec = request->spec;
+    std::string name{spec.surface_name.is_set() ?
+                     spec.surface_name.value() : ""};
 
-    try
+    std::shared_ptr<mcl::ClientBufferStream> default_stream {nullptr};
+    if (surface_proto->buffer_stream().has_id())
     {
-        std::string name{spec.surface_name.is_set() ?
-                         spec.surface_name.value() : ""};
-
-        stream = std::make_shared<mcl::BufferStream>(
-            this, request->wh, server, platform,
-            surface_proto->buffer_stream(), make_perf_report(logger), name,
-            mir::geometry::Size{surface_proto->width(), surface_proto->height()}, nbuffers);
-    }
-    catch (std::exception const& error)
-    {
-        if (!surface_proto->has_error())
-            surface_proto->set_error(error.what());
-        // failed to create buffer_stream, so clean up FDs it doesn't own
-        for (auto i = 0; i < surface_proto->fd_size(); i++)
-            ::close(surface_proto->fd(i));
-        if (surface_proto->has_buffer_stream() && surface_proto->buffer_stream().has_buffer())
-            for (int i = 0; i < surface_proto->buffer_stream().buffer().fd_size(); i++)
-                ::close(surface_proto->buffer_stream().buffer().fd(i));
+        try
+        {
+            default_stream = std::make_shared<mcl::BufferStream>(
+                this, request->wh, server, platform, surface_map, buffer_factory,
+                surface_proto->buffer_stream(), make_perf_report(logger), name,
+                mir::geometry::Size{surface_proto->width(), surface_proto->height()}, nbuffers);
+            surface_map->insert(default_stream->rpc_id(), default_stream);
+        }
+        catch (std::exception const& error)
+        {
+            if (!surface_proto->has_error())
+                surface_proto->set_error(error.what());
+            // failed to create buffer_stream, so clean up FDs it doesn't own
+            for (auto i = 0; i < surface_proto->fd_size(); i++)
+                ::close(surface_proto->fd(i));
+            if (surface_proto->has_buffer_stream() && surface_proto->buffer_stream().has_buffer())
+                for (int i = 0; i < surface_proto->buffer_stream().buffer().fd_size(); i++)
+                    ::close(surface_proto->buffer_stream().buffer().fd(i));
+        }
     }
 
+    std::shared_ptr<MirSurface> surf {nullptr};
     if (surface_proto->has_error() || !surface_proto->has_id())
     {
         std::string reason;
@@ -397,10 +418,8 @@ void MirConnection::surface_created(SurfaceCreationRequest* request)
     else
     {
         surf = std::make_shared<MirSurface>(
-            this, server, &debug, stream, input_platform, spec, *surface_proto, request->wh);
-
+            this, server, &debug, default_stream, input_platform, spec, *surface_proto, request->wh);
         surface_map->insert(mf::SurfaceId{surface_proto->id().value()}, surf);
-        surface_map->insert(mf::BufferStreamId{surface_proto->id().value()}, stream);
     }
 
     callback(surf.get(), context);
@@ -457,13 +476,6 @@ void MirConnection::released(StreamRelease data)
 
 void MirConnection::released(SurfaceRelease data)
 {
-    // releasing this surface from surface_map means that it will no longer receive events
-    // If it's still focused, send an unfocused event before we kill it entirely
-    if (data.surface->attrib(mir_surface_attrib_focus) == mir_surface_focused)
-    {
-        auto unfocus = mev::make_event(mir::frontend::SurfaceId{data.surface->id()}, mir_surface_attrib_focus, mir_surface_unfocused);
-        data.surface->handle_event(*unfocus);
-    }
     data.callback(data.surface, data.context);
     data.handle->result_received();
     surface_map->erase(mf::BufferStreamId(data.surface->id()));
@@ -611,7 +623,7 @@ MirWaitHandle* MirConnection::disconnect()
         std::lock_guard<decltype(mutex)> lock(mutex);
         disconnecting = true;
     }
-    surface_map->with_all_streams_do([](mcl::BufferReceiver* receiver)
+    surface_map->with_all_streams_do([](mcl::ClientBufferStream* receiver)
     {
         receiver->buffer_unavailable();
     });
@@ -810,7 +822,7 @@ void MirConnection::stream_created(StreamCreationRequest* request_raw)
     try
     {
         auto stream = std::make_shared<mcl::BufferStream>(
-            this, request->wh, server, platform,
+            this, request->wh, server, platform, surface_map, buffer_factory,
             *protobuf_bs, make_perf_report(logger), std::string{},
             mir::geometry::Size{request->parameters.width(), request->parameters.height()}, nbuffers);
         surface_map->insert(mf::BufferStreamId(protobuf_bs->id().value()), stream);
@@ -879,7 +891,7 @@ void MirConnection::stream_error(std::string const& error_msg, std::shared_ptr<S
 }
 
 std::shared_ptr<mir::client::ClientBufferStream> MirConnection::make_consumer_stream(
-   mp::BufferStream const& protobuf_bs, mir::geometry::Size)
+   mp::BufferStream const& protobuf_bs)
 {
     return std::make_shared<mcl::ScreencastStream>(
         this, server, platform, protobuf_bs);
@@ -908,6 +920,11 @@ void MirConnection::register_ping_event_callback(mir_ping_event_callback callbac
     ping_handler->set_callback(std::bind(callback, this, std::placeholders::_1, context));
 }
 
+void MirConnection::register_error_callback(mir_error_callback callback, void* context)
+{
+    error_handler.set_callback(std::bind(callback, this, std::placeholders::_1, context));
+}
+
 void MirConnection::pong(int32_t serial)
 {
     mp::PingEvent pong;
@@ -991,6 +1008,58 @@ MirWaitHandle* MirConnection::set_base_display_configuration(MirDisplayConfigura
     return &set_base_display_configuration_wait_handle;
 }
 
+namespace
+{
+struct HandleErrorVoid
+{
+    std::unique_ptr<mp::Void> result;
+    std::function<void(mp::Void const&)> on_error;
+};
+
+void handle_structured_error(HandleErrorVoid* handler)
+{
+    if (handler->result->has_structured_error())
+    {
+        handler->on_error(*handler->result);
+    }
+    delete handler;
+}
+}
+
+void MirConnection::preview_base_display_configuration(
+    mp::DisplayConfiguration const& configuration,
+    std::chrono::seconds timeout)
+{
+    mp::PreviewConfiguration request;
+
+    request.mutable_configuration()->CopyFrom(configuration);
+    request.set_timeout(timeout.count());
+
+    auto store_error_result = new HandleErrorVoid;
+    store_error_result->result = std::make_unique<mp::Void>();
+    store_error_result->on_error = [this](mp::Void const& message)
+    {
+        MirError const error{
+            static_cast<MirErrorDomain>(message.structured_error().domain()),
+            message.structured_error().code()};
+        error_handler(&error);
+    };
+
+    server.preview_base_display_configuration(
+        &request,
+        store_error_result->result.get(),
+        google::protobuf::NewCallback(&handle_structured_error, store_error_result));
+}
+
+void MirConnection::confirm_base_display_configuration(
+    mp::DisplayConfiguration const& configuration)
+{
+    server.confirm_base_display_configuration(
+        &configuration,
+        set_base_display_configuration_response.get(),
+        google::protobuf::NewCallback(google::protobuf::DoNothing));
+}
+
 void MirConnection::done_set_base_display_configuration()
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
@@ -1164,19 +1233,22 @@ void MirConnection::allocate_buffer(
     if (!client_buffer_factory)
         client_buffer_factory = platform->create_buffer_factory();
     buffer_factory->expect_buffer(
-        client_buffer_factory,
-        nullptr,
+        client_buffer_factory, this,
         size, format, usage,
         callback, context);
     server.allocate_buffers(&request, ignored.get(), gp::NewCallback(ignore));
 }
 
-void MirConnection::release_buffer(int buffer_id)
+void MirConnection::release_buffer(mcl::MirBuffer* buffer)
 {
-    surface_map->erase(buffer_id);
+    if (!buffer->valid())
+    {
+        surface_map->erase(buffer->rpc_id());
+        return;
+    }
 
     mp::BufferRelease request;
     auto released_buffer = request.add_buffers();
-    released_buffer->set_buffer_id(buffer_id);
+    released_buffer->set_buffer_id(buffer->rpc_id());
     server.release_buffers(&request, ignored.get(), gp::NewCallback(ignore));
 }
