@@ -33,7 +33,7 @@
 #include "mir/test/doubles/stub_session_authorizer.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/pipe.h"
-#include "mir/test/wait_condition.h"
+#include "mir/test/signal.h"
 #include "mir/test/signal.h"
 
 #include "mir_toolkit/mir_client_library.h"
@@ -95,12 +95,12 @@ struct StubAuthorizer : mtd::StubSessionAuthorizer
 
 void wait_for_server_actions_to_finish(mir::ServerActionQueue& server_action_queue)
 {
-    mt::WaitCondition last_action_done;
+    mt::Signal last_action_done;
     server_action_queue.enqueue(
         &last_action_done,
-        [&] { last_action_done.wake_up_everyone(); });
+        [&] { last_action_done.raise(); });
 
-    last_action_done.wait_for_at_most_seconds(5);
+    last_action_done.wait_for(std::chrono::seconds{5});
 }
 }
 
@@ -764,3 +764,209 @@ TEST_F(DisplayConfigurationTest, client_sees_server_set_form_factor)
 
     client.disconnect();
 }
+
+TEST_F(DisplayConfigurationTest, preview_base_display_configuration_sends_config_event)
+{
+    DisplayClient client{new_connection()};
+
+    client.connect();
+
+    std::shared_ptr<MirDisplayConfig> config = client.get_base_config();
+
+    for (auto i = 0; i < mir_display_config_get_num_outputs(config.get()); ++i)
+    {
+        auto output = mir_display_config_get_mutable_output(config.get(), i);
+
+        for (auto j = 0; j < mir_output_get_num_modes(output); ++j)
+        {
+            auto mode = mir_output_get_mode(output, j);
+
+            if (mode != mir_output_get_current_mode(output))
+            {
+                mir_output_set_current_mode(output, mode);
+                break;
+            }
+        }
+    }
+
+    ASSERT_THAT(config.get(), Not(mt::DisplayConfigMatches(client.get_base_config().get())));
+
+    DisplayConfigMatchingContext context;
+    context.matcher = [config](MirDisplayConfig* conf)
+        {
+            EXPECT_THAT(conf, mt::DisplayConfigMatches(config.get()));
+        };
+
+    mir_connection_set_display_config_change_callback(
+        client.connection,
+        &new_display_config_matches,
+        &context);
+
+    mir_connection_preview_base_display_configuration(client.connection, config.get(), 5);
+
+    EXPECT_TRUE(context.done.wait_for(std::chrono::seconds{10}));
+
+    client.disconnect();
+}
+
+TEST_F(DisplayConfigurationTest, preview_base_display_configuration_reverts_after_timeout)
+{
+    DisplayClient client{new_connection()};
+
+    client.connect();
+
+    std::shared_ptr<MirDisplayConfig> old_config = client.get_base_config();
+    std::shared_ptr<MirDisplayConfig> new_config = client.get_base_config();
+
+    for (auto i = 0; i < mir_display_config_get_num_outputs(new_config.get()); ++i)
+    {
+        auto output = mir_display_config_get_mutable_output(new_config.get(), i);
+
+        for (auto j = 0; j < mir_output_get_num_modes(output); ++j)
+        {
+            auto mode = mir_output_get_mode(output, j);
+
+            if (mode != mir_output_get_current_mode(output))
+            {
+                mir_output_set_current_mode(output, mode);
+                break;
+            }
+        }
+    }
+
+    ASSERT_THAT(new_config.get(), Not(mt::DisplayConfigMatches(old_config.get())));
+
+    DisplayConfigMatchingContext context;
+    auto reverted = std::make_shared<mt::Signal>();
+    context.matcher = [old_config, new_config, reverted](MirDisplayConfig* conf)
+        {
+            static int call_count{0};
+            ++call_count;
+            if (call_count == 1)
+            {
+                EXPECT_THAT(conf, mt::DisplayConfigMatches(new_config.get()));
+            }
+            else if (call_count == 2)
+            {
+                EXPECT_THAT(conf, mt::DisplayConfigMatches(old_config.get()));
+                reverted->raise();
+            }
+            else
+            {
+                FAIL() << "Received unexpected configuration event";
+            }
+        };
+
+    mir_connection_set_display_config_change_callback(
+        client.connection,
+        &new_display_config_matches,
+        &context);
+
+    mir_connection_preview_base_display_configuration(client.connection, new_config.get(), 5);
+
+    std::this_thread::sleep_for(std::chrono::seconds{3});
+    // Should still have the old config
+    EXPECT_TRUE(context.done.raised());
+    EXPECT_FALSE(reverted->raised());
+
+    EXPECT_TRUE(reverted->wait_for(std::chrono::seconds{10}));
+
+    client.disconnect();
+}
+
+TEST_F(DisplayConfigurationTest, display_configuration_sticks_after_confirmation)
+{
+    DisplayClient client{new_connection()};
+
+    client.connect();
+
+    std::shared_ptr<MirDisplayConfig> old_config = client.get_base_config();
+    std::shared_ptr<MirDisplayConfig> new_config = client.get_base_config();
+
+    for (auto i = 0; i < mir_display_config_get_num_outputs(new_config.get()); ++i)
+    {
+        auto output = mir_display_config_get_mutable_output(new_config.get(), i);
+
+        for (auto j = 0; j < mir_output_get_num_modes(output); ++j)
+        {
+            auto mode = mir_output_get_mode(output, j);
+
+            if (mode != mir_output_get_current_mode(output))
+            {
+                mir_output_set_current_mode(output, mode);
+                break;
+            }
+        }
+    }
+
+    ASSERT_THAT(new_config.get(), Not(mt::DisplayConfigMatches(old_config.get())));
+
+    DisplayConfigMatchingContext context;
+    auto signalled_twice = std::make_shared<mt::Signal>();
+    context.matcher = [new_config, signalled_twice](MirDisplayConfig* conf)
+        {
+            static int call_count{0};
+            ++call_count;
+            EXPECT_THAT(conf, mt::DisplayConfigMatches(new_config.get()));
+            if (call_count == 2)
+            {
+                signalled_twice->raise();
+            }
+        };
+
+    mir_connection_set_display_config_change_callback(
+        client.connection,
+        &new_display_config_matches,
+        &context);
+
+    mir_connection_preview_base_display_configuration(client.connection, new_config.get(), 10);
+
+    EXPECT_TRUE(context.done.wait_for(std::chrono::seconds{5}));
+
+    mir_connection_confirm_base_display_configuration(client.connection, new_config.get());
+
+    EXPECT_TRUE(signalled_twice->wait_for(std::chrono::seconds{10}));
+
+    client.disconnect();
+}
+
+namespace
+{
+struct ErrorValidator
+{
+    mt::Signal received;
+    std::function<void(MirError const*)> validate;
+};
+
+void validating_error_handler(MirConnection*, MirError const* error, void* context)
+{
+    auto& error_validator = *reinterpret_cast<ErrorValidator*>(context);
+    error_validator.validate(error);
+    error_validator.received.raise();
+}
+}
+
+TEST_F(DisplayConfigurationTest, unauthorised_client_receives_error)
+{
+    stub_authorizer.allow_set_base_display_configuration = false;
+
+    DisplayClient client{new_connection()};
+
+    client.connect();
+
+    auto config = client.get_base_config();
+
+    ErrorValidator validator;
+    validator.validate = [&config](MirError const* error)
+        {
+            EXPECT_THAT(mir_error_get_domain(error), Eq(mir_error_domain_display_configuration));
+            EXPECT_THAT(mir_error_get_code(error), Eq(mir_display_configuration_error_unauthorized));
+        };
+    mir_connection_set_error_callback(client.connection, &validating_error_handler, &validator);
+
+    mir_connection_preview_base_display_configuration(client.connection, config.get(), 20);
+
+    EXPECT_TRUE(validator.received.wait_for(std::chrono::seconds{10}));
+
+    client.disconnect();
+}
