@@ -19,6 +19,7 @@
 #include "mir/events/event_builders.h"
 #include "mir/scene/surface.h"
 
+#include "mir_test_framework/temporary_environment_value.h"
 #include "mir/test/doubles/wrap_shell_to_track_latest_surface.h"
 #include "mir_test_framework/connected_client_headless_server.h"
 #include "mir/test/fake_shared.h"
@@ -122,8 +123,8 @@ struct SurfacePlacement : mtf::ConnectedClientHeadlessServer
         auto const modifiers = mir_input_event_modifier_none;
         auto const depressed_buttons = mir_pointer_button_primary;
 
-        auto const x_axis_value = click_position.x.as_float();
-        auto const y_axis_value = click_position.y.as_float();
+        auto const x_axis_value = click_position.x.as_int();
+        auto const y_axis_value = click_position.y.as_int();
         auto const hscroll_value = 0.0;
         auto const vscroll_value = 0.0;
         auto const action = mir_pointer_action_button_down;
