@@ -30,7 +30,9 @@
 #include "mir/geometry/rectangle.h"
 #include "mir/test/event_matchers.h"
 #include "mir/test/doubles/mock_libinput.h"
+#include "mir/test/doubles/mock_input_seat.h"
 #include "mir/test/gmock_fixes.h"
+#include "mir/test/fake_shared.h"
 #include "mir/udev/wrapper.h"
 #include "mir/cookie/authority.h"
 #include "mir_test_framework/libinput_environment.h"
@@ -71,12 +73,15 @@ struct MockInputSink : mi::InputSink
     MOCK_METHOD1(handle_input,void(MirEvent &));
     MOCK_METHOD1(confine_pointer, void(geom::Point&));
     MOCK_CONST_METHOD0(bounding_rectangle, geom::Rectangle());
+    MOCK_METHOD1(key_state, void(std::vector<uint32_t> const&));
+    MOCK_METHOD1(pointer_state, void(MirPointerButtons));
 };
 
 struct MockEventBuilder : mi::EventBuilder
 {
     std::shared_ptr<mir::cookie::Authority> const cookie_authority = mir::cookie::Authority::create();
-    mi::DefaultEventBuilder builder{MirInputDeviceId{3}, cookie_authority};
+    mtd::MockInputSeat seat;
+    mi::DefaultEventBuilder builder{MirInputDeviceId{3}, cookie_authority, mt::fake_shared(seat)};
     MockEventBuilder()
     {
         ON_CALL(*this, key_event(_,_,_,_))
@@ -119,6 +124,10 @@ struct MockEventBuilder : mi::EventBuilder
     MOCK_METHOD7(pointer_event,
                  mir::EventUPtr(Timestamp, MirPointerAction, MirPointerButtons, float, float, float, float));
     MOCK_METHOD2(configuration_event, mir::EventUPtr(Timestamp, MirInputConfigurationAction));
+    mir::EventUPtr device_state_event(float, float) override
+    {
+        return {nullptr,[](MirEvent*){}};
+    }
 };
 
 struct LibInputDevice : public ::testing::Test
@@ -407,6 +416,20 @@ TEST_F(LibInputDeviceOnMouse, process_event_motion_events_with_relative_changes)
     process_events(mouse);
 }
 
+TEST_F(LibInputDeviceOnMouse, notices_slow_relative_movements)
+{   // Regression test for LP: #1528109
+    float x1 = 0.5f, x2 = 0.6f;
+    float y1 = 1.7f, y2 = 1.8f;
+
+    EXPECT_CALL(mock_sink, handle_input(mt::PointerEventWithDiff(x1,y1)));
+    EXPECT_CALL(mock_sink, handle_input(mt::PointerEventWithDiff(x2,y2)));
+
+    mouse.start(&mock_sink, &mock_builder);
+    env.mock_libinput.setup_pointer_event(fake_device, event_time_1, x1, y1);
+    env.mock_libinput.setup_pointer_event(fake_device, event_time_2, x2, y2);
+    process_events(mouse);
+}
+
 TEST_F(LibInputDeviceOnMouse, process_event_handles_press_and_release)
 {
     float const x = 0;
