@@ -264,7 +264,7 @@ TEST_F(SurfaceInputDispatcher, key_event_delivered_to_focused_surface)
     FakeKeyboard keyboard;
     auto event = keyboard.press();
 
-    EXPECT_CALL(*surface, consume(mt::MirKeyboardEventMatches(*event))).Times(1);
+    EXPECT_CALL(*surface, consume(mt::MirKeyboardEventMatches(ByRef(*event)))).Times(1);
 
     dispatcher.start();
 
@@ -284,97 +284,6 @@ TEST_F(SurfaceInputDispatcher, key_event_dropped_if_no_surface_focused)
     EXPECT_FALSE(dispatcher.dispatch(*keyboard.press()));
 }
 
-TEST_F(SurfaceInputDispatcher, inconsistent_key_events_dropped)
-{
-    auto surface = scene.add_surface();
-
-    EXPECT_CALL(*surface, consume(_)).Times(0);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface);
-
-    FakeKeyboard keyboard;
-    EXPECT_FALSE(dispatcher.dispatch(*keyboard.release()));
-}
-
-TEST_F(SurfaceInputDispatcher, key_state_is_consistent_per_client)
-{
-    auto surface_1 = scene.add_surface();
-    auto surface_2 = scene.add_surface();
-
-    FakeKeyboard keyboard;
-    auto down_event = keyboard.press();
-    auto up_event = keyboard.release();
-
-    EXPECT_CALL(*surface_1, consume(mt::MirKeyboardEventMatches(*down_event))).Times(1);
-    EXPECT_CALL(*surface_2, consume(_)).Times(0);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface_1);
-    EXPECT_TRUE(dispatcher.dispatch(*down_event));
-    dispatcher.set_focus(surface_2);
-    EXPECT_FALSE(dispatcher.dispatch(*up_event));
-}
-
-TEST_F(SurfaceInputDispatcher, inconsistent_key_down_dropped)
-{
-    auto surface = scene.add_surface();
-
-    FakeKeyboard keyboard;
-    auto event = keyboard.press();
-
-    InSequence seq;
-    EXPECT_CALL(*surface, consume(mt::MirKeyboardEventMatches(*event))).Times(1);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface);
-    EXPECT_TRUE(dispatcher.dispatch(*event));
-    EXPECT_FALSE(dispatcher.dispatch(*event));
-    EXPECT_FALSE(dispatcher.dispatch(*event));
-}
-
-TEST_F(SurfaceInputDispatcher, device_reset_resets_key_state_consistency)
-{
-    auto surface = scene.add_surface();
-
-    auto device_id = MirInputDeviceId{1};
-    FakeKeyboard keyboard(device_id);
-    auto down_event = keyboard.press(11);
-    auto release_event = keyboard.release(11);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface);
-    EXPECT_TRUE(dispatcher.dispatch(*down_event));
-    EXPECT_TRUE(dispatcher.dispatch(
-        *mev::make_event(mir_input_configuration_action_device_reset, device_id, std::chrono::nanoseconds{1})));
-    EXPECT_FALSE(dispatcher.dispatch(*release_event));
-}
-
-TEST_F(SurfaceInputDispatcher, key_input_target_may_disappear_and_things_remain_quote_a_unquote_ok)
-{
-    auto surface_2 = scene.add_surface();
-    auto surface_1 = scene.add_surface();
-
-    EXPECT_CALL(*surface_1, consume(_)).Times(AnyNumber());
-    EXPECT_CALL(*surface_2, consume(_)).Times(AnyNumber());
-
-    FakeKeyboard k;
-    auto an_ev = k.press(11);
-    auto another_ev = k.press(12);
-
-    dispatcher.start();
-    dispatcher.set_focus(surface_1);
-    EXPECT_TRUE(dispatcher.dispatch(*an_ev));
-    scene.remove_surface(surface_1);
-    EXPECT_FALSE(dispatcher.dispatch(*another_ev));
-    dispatcher.set_focus(surface_2);
-    EXPECT_TRUE(dispatcher.dispatch(*another_ev));
-}
-
 TEST_F(SurfaceInputDispatcher, pointer_motion_delivered_to_client_under_pointer)
 {
     auto surface = scene.add_surface({{0, 0}, {5, 5}});
