@@ -35,6 +35,7 @@
 #include "mir/test/doubles/mock_event_sink.h"
 #include "mir/test/doubles/null_prompt_session.h"
 #include "mir/test/doubles/stub_display_configuration.h"
+#include "mir/test/doubles/stub_buffer_allocator.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -61,10 +62,17 @@ static std::shared_ptr<mtd::MockSurface> make_mock_surface()
 
 struct MockBufferStreamFactory : public ms::BufferStreamFactory
 {
+    MockBufferStreamFactory()
+    {
+        ON_CALL(*this, create_buffer_map(testing::_))
+            .WillByDefault(testing::Return(std::make_shared<mtd::StubClientBuffers>()));
+    }
     MOCK_METHOD3(create_buffer_stream, std::shared_ptr<mc::BufferStream>(
-        mf::BufferStreamId, std::shared_ptr<mf::BufferSink> const&, mg::BufferProperties const&));
+        mf::BufferStreamId, std::shared_ptr<mf::ClientBuffers> const&, mg::BufferProperties const&));
     MOCK_METHOD4(create_buffer_stream, std::shared_ptr<mc::BufferStream>(
-        mf::BufferStreamId, std::shared_ptr<mf::BufferSink> const&, int, mg::BufferProperties const&));
+        mf::BufferStreamId, std::shared_ptr<mf::ClientBuffers> const&, int, mg::BufferProperties const&));
+    MOCK_METHOD1(create_buffer_map, std::shared_ptr<mf::ClientBuffers>(
+        std::shared_ptr<mf::BufferSink> const&));
 };
 
 
@@ -95,7 +103,7 @@ MATCHER(IsNullSnapshot, "")
 }
 
 MATCHER_P(EqPromptSessionEventState, state, "") {
-  return arg.type == mir_event_type_prompt_session_state_change && arg.prompt_session.new_state == state;
+  return arg.type() == mir_event_type_prompt_session_state_change && arg.to_prompt_session()->new_state() == state;
 }
 
 MATCHER_P(HasParent, parent, "")
@@ -148,7 +156,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            stub_session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
     
     std::shared_ptr<ms::ApplicationSession> make_application_session(
@@ -161,7 +169,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            stub_session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
 
     std::shared_ptr<ms::ApplicationSession> make_application_session(
@@ -174,7 +182,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            stub_session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
     std::shared_ptr<ms::ApplicationSession> make_application_session_with_coordinator(
         std::shared_ptr<msh::SurfaceStack> const& surface_stack)
@@ -185,7 +193,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            stub_session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
     
     std::shared_ptr<ms::ApplicationSession> make_application_session_with_listener(
@@ -197,7 +205,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
 
 
@@ -210,7 +218,7 @@ struct ApplicationSession : public testing::Test
            null_snapshot_strategy,
            stub_session_listener,
            mtd::StubDisplayConfig{},
-           event_sink);
+           event_sink, allocator);
     }
 
     std::shared_ptr<mtd::NullEventSink> const event_sink;
@@ -221,14 +229,17 @@ struct ApplicationSession : public testing::Test
         std::make_shared<mtd::StubBufferStreamFactory>();
     std::shared_ptr<mtd::StubSurfaceFactory> const stub_surface_factory{std::make_shared<mtd::StubSurfaceFactory>()};
     std::shared_ptr<mtd::StubBufferStream> const stub_buffer_stream{std::make_shared<mtd::StubBufferStream>()};
+    std::shared_ptr<mtd::StubBufferAllocator> const allocator{
+        std::make_shared<mtd::StubBufferAllocator>()};
     pid_t pid;
     std::string name;
+    mg::BufferProperties properties { geom::Size{1,1}, mir_pixel_format_abgr_8888, mg::BufferUsage::hardware };
 };
 
 struct MockSurfaceFactory : ms::SurfaceFactory
 {
     MOCK_METHOD2(create_surface, std::shared_ptr<ms::Surface>(
-        std::shared_ptr<mc::BufferStream> const&, ms::SurfaceCreationParameters const& params));
+        std::list<ms::StreamInfo> const&, ms::SurfaceCreationParameters const& params));
 };
 }
 
@@ -246,12 +257,50 @@ TEST_F(ApplicationSession, adds_created_surface_to_coordinator)
     auto session = make_application_session(
         mt::fake_shared(surface_stack), mt::fake_shared(mock_surface_factory));
 
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(session->create_buffer_stream(properties));
     auto surf = session->create_surface(params, event_sink);
 
     session->destroy_surface(surf);
 }
 
+TEST_F(ApplicationSession, attempt_to_destroy_non_existent_stream_throws)
+{
+    using namespace ::testing;
+    NiceMock<MockSurfaceFactory> mock_surface_factory;
+    NiceMock<mtd::MockSurfaceStack> surface_stack;
+    auto session = make_application_session(
+        mt::fake_shared(surface_stack), mt::fake_shared(mock_surface_factory));
+
+    mf::BufferStreamId made_up_id{332};
+    EXPECT_THROW({
+        session->destroy_buffer_stream(made_up_id);
+    }, std::runtime_error);
+}
+
+TEST_F(ApplicationSession, can_destroy_buffer_stream_after_destroying_surface)
+{
+    using namespace ::testing;
+
+    NiceMock<MockSurfaceFactory> mock_surface_factory;
+    NiceMock<mtd::MockSurfaceStack> surface_stack;
+    std::shared_ptr<ms::Surface> mock_surface = make_mock_surface();
+
+    EXPECT_CALL(mock_surface_factory, create_surface(_,_))
+        .WillOnce(Return(mock_surface));
+    EXPECT_CALL(surface_stack, add_surface(mock_surface,_));
+    auto session = make_application_session(
+        mt::fake_shared(surface_stack), mt::fake_shared(mock_surface_factory));
+
+    auto buffer_stream = session->create_buffer_stream(properties);
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(buffer_stream);
+    auto surf = session->create_surface(params, event_sink);
+
+    session->destroy_surface(surf);
+    session->destroy_buffer_stream(buffer_stream);
+}
+
 TEST_F(ApplicationSession, notifies_listener_of_create_and_destroy_surface)
 {
     using namespace ::testing;
@@ -264,7 +313,8 @@ TEST_F(ApplicationSession, notifies_listener_of_create_and_destroy_surface)
 
     auto session = make_application_session_with_listener(mt::fake_shared(listener));
 
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(session->create_buffer_stream(properties));
     auto surf = session->create_surface(params, event_sink);
 
     session->destroy_surface(surf);
@@ -283,7 +333,8 @@ TEST_F(ApplicationSession, notifies_listener_of_surface_destruction_via_session_
     {
         auto session = make_application_session_with_listener(mt::fake_shared(listener));
 
-        ms::SurfaceCreationParameters params;
+        ms::SurfaceCreationParameters params = ms::a_surface()
+            .with_buffer_stream(session->create_buffer_stream(properties));
         session->create_surface(params, event_sink);
     }
 }
@@ -320,7 +371,8 @@ TEST_F(ApplicationSession, default_surface_is_first_surface)
 
     auto app_session = make_application_session_with_stubs();
 
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(app_session->create_buffer_stream(properties));
     auto id1 = app_session->create_surface(params, nullptr);
     auto id2 = app_session->create_surface(params, nullptr);
     auto id3 = app_session->create_surface(params, nullptr);
@@ -341,7 +393,8 @@ TEST_F(ApplicationSession, default_surface_is_first_surface)
 TEST_F(ApplicationSession, foreign_surface_has_no_successor)
 {
     auto session1 = make_application_session_with_stubs();
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(session1->create_buffer_stream(properties));
     auto id1 = session1->create_surface(params, nullptr);
     auto surf1 = session1->surface(id1);
     auto id2 = session1->create_surface(params, nullptr);
@@ -358,7 +411,8 @@ TEST_F(ApplicationSession, foreign_surface_has_no_successor)
 TEST_F(ApplicationSession, surface_after_one_is_self)
 {
     auto session = make_application_session_with_stubs();
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(session->create_buffer_stream(properties));
     auto id = session->create_surface(params, nullptr);
     auto surf = session->surface(id);
 
@@ -371,7 +425,8 @@ TEST_F(ApplicationSession, surface_after_cycles_through_all)
 {
     auto app_session = make_application_session_with_stubs();
 
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(app_session->create_buffer_stream(properties));
 
     int const N = 3;
     std::shared_ptr<ms::Surface> surf[N];
@@ -412,7 +467,8 @@ TEST_F(ApplicationSession, session_visbility_propagates_to_surfaces)
         EXPECT_CALL(*mock_surface, show()).Times(1);
     }
 
-    ms::SurfaceCreationParameters params;
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(app_session->create_buffer_stream(properties));
     auto surf = app_session->create_surface(params, event_sink);
 
     app_session->hide();
@@ -445,9 +501,11 @@ TEST_F(ApplicationSession, takes_snapshot_of_default_surface)
         snapshot_strategy,
         std::make_shared<ms::NullSessionListener>(),
         mtd::StubDisplayConfig{},
-        event_sink);
+        event_sink, allocator);
 
-    auto surface = app_session.create_surface(ms::SurfaceCreationParameters{}, event_sink);
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
+    auto surface = app_session.create_surface(params, event_sink);
     app_session.take_snapshot(ms::SnapshotCallback());
     app_session.destroy_surface(surface);
 }
@@ -466,7 +524,7 @@ TEST_F(ApplicationSession, returns_null_snapshot_if_no_default_surface)
         snapshot_strategy,
         std::make_shared<ms::NullSessionListener>(),
         mtd::StubDisplayConfig{},
-        event_sink);
+        event_sink, allocator);
 
     EXPECT_CALL(*snapshot_strategy, take_snapshot_of(_,_)).Times(0);
     EXPECT_CALL(mock_snapshot_callback, operator_call(IsNullSnapshot()));
@@ -487,47 +545,18 @@ TEST_F(ApplicationSession, process_id)
         null_snapshot_strategy,
         std::make_shared<ms::NullSessionListener>(),
         mtd::StubDisplayConfig{},
-        event_sink);
+        event_sink, allocator);
 
     EXPECT_THAT(app_session.process_id(), Eq(session_pid));
 }
 
-TEST_F(ApplicationSession, surface_ids_are_bufferstream_ids)
-{
-    using namespace ::testing;
-
-    NiceMock<MockSurfaceFactory> mock_surface_factory;
-    NiceMock<MockBufferStreamFactory> mock_bufferstream_factory;
-    NiceMock<mtd::MockSurfaceStack> surface_stack;
-    std::shared_ptr<ms::Surface> mock_surface = make_mock_surface();
-    auto stub_bstream = std::make_shared<mtd::StubBufferStream>();
-    EXPECT_CALL(mock_bufferstream_factory, create_buffer_stream(_,_,_))
-        .WillOnce(Return(stub_bstream));
-    EXPECT_CALL(mock_surface_factory, create_surface(std::shared_ptr<mc::BufferStream>(stub_bstream),_))
-        .WillOnce(Return(mock_surface));
-    auto session = make_application_session(
-        mt::fake_shared(mock_bufferstream_factory),
-        mt::fake_shared(mock_surface_factory));
-
-    ms::SurfaceCreationParameters params;
-
-    auto id1 = session->create_surface(params, event_sink);
-    EXPECT_THAT(session->get_buffer_stream(mf::BufferStreamId(id1.as_value())), Eq(stub_bstream));
-    EXPECT_THAT(session->get_surface(id1), Eq(mock_surface));
-
-    session->destroy_surface(id1);
-
-    EXPECT_THROW({
-            session->get_buffer_stream(mf::BufferStreamId(id1.as_value()));
-    }, std::runtime_error);
-}
-
 TEST_F(ApplicationSession, can_destroy_surface_bstream)
 {
     auto session = make_application_session_with_stubs();
-    ms::SurfaceCreationParameters params;
+    mf::BufferStreamId stream_id = session->create_buffer_stream(properties);
+    ms::SurfaceCreationParameters params = ms::a_surface()
+        .with_buffer_stream(stream_id);
     auto id = session->create_surface(params, event_sink);
-    mf::BufferStreamId stream_id(id.as_value());
     session->destroy_buffer_stream(stream_id);
     EXPECT_THROW({
         session->get_buffer_stream(stream_id);
@@ -565,24 +594,25 @@ TEST_F(ApplicationSession, sets_and_looks_up_surface_streams)
     auto session = make_application_session(
         mt::fake_shared(mock_bufferstream_factory),
         mt::fake_shared(mock_surface_factory));
-    auto stream_id0 = mf::BufferStreamId(
-        session->create_surface(
-            ms::a_surface().of_position({1,1}),
-            event_sink).as_value());
 
+    auto stream_id0 = session->create_buffer_stream(stream_properties);
     auto stream_id1 = session->create_buffer_stream(stream_properties);
     auto stream_id2 = session->create_buffer_stream(stream_properties);
 
     std::list<ms::StreamInfo> info {
-        {streams[2], geom::Displacement{0,3}},
-        {streams[0], geom::Displacement{-1,1}},
-        {streams[1], geom::Displacement{0,2}}
+        {streams[2], geom::Displacement{0,3}, {}},
+        {streams[0], geom::Displacement{-1,1}, {}},
+        {streams[1], geom::Displacement{0,2}, {}}
     };
+
+    session->create_surface(
+        ms::a_surface().with_buffer_stream(stream_id0), event_sink);
+
     EXPECT_CALL(*mock_surface, set_streams(Pointwise(StreamEq(), info)));
     session->configure_streams(*mock_surface, {
-        {stream_id2, geom::Displacement{0,3}},
-        {stream_id0, geom::Displacement{-1,1}},
-        {stream_id1, geom::Displacement{0,2}}
+        {stream_id2, geom::Displacement{0,3}, {}},
+        {stream_id0, geom::Displacement{-1,1}, {}},
+        {stream_id1, geom::Displacement{0,2}, {}}
     });
 }
 
@@ -633,6 +663,15 @@ TEST_F(ApplicationSession, buffer_stream_constructed_with_swapinterval_1)
     session->destroy_buffer_stream(id);
 }
 
+MATCHER_P(HasSingleStream, value, "")
+{
+    using namespace testing;
+    EXPECT_THAT(arg.size(), Eq(1));
+    if (arg.size() < 1 ) return false;
+    EXPECT_THAT(arg.front().stream.get(), Eq(value.get())); 
+    return !(::testing::Test::HasFailure());
+}
+
 TEST_F(ApplicationSession, surface_uses_prexisting_buffer_stream_if_set)
 {
     using namespace testing;
@@ -649,25 +688,17 @@ TEST_F(ApplicationSession, surface_uses_prexisting_buffer_stream_if_set)
         mt::fake_shared(mock_surface_factory));
 
     auto id = session->create_buffer_stream(properties);
+    auto stream = session->get_buffer_stream(id);
 
-    EXPECT_CALL(mock_surface_factory, create_surface(Eq(session->get_buffer_stream(id)),_))
-        .WillOnce(Invoke([&](auto bs, auto)
-    {
-        auto surface = make_mock_surface();
-        ON_CALL(*surface, primary_buffer_stream())
-            .WillByDefault(Return(bs));
-        return surface;
-    }));
+    EXPECT_CALL(mock_surface_factory, create_surface(HasSingleStream(stream),_))
+        .WillOnce(Return(make_mock_surface()));
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
         .of_name("Aardavks")
         .of_type(mir_surface_type_normal)
         .with_buffer_stream(id);
 
-    auto surface_id = session->create_surface(params, event_sink);
-    auto surface = session->get_surface(surface_id);
-
-    EXPECT_THAT(surface->primary_buffer_stream(), Eq(session->get_buffer_stream(id)));
+    session->create_surface(params, event_sink);
 }
 
 namespace
@@ -684,7 +715,8 @@ struct ApplicationSessionSender : public ApplicationSession
             null_snapshot_strategy,
             stub_session_listener,
             mtd::StubDisplayConfig{},
-            mt::fake_shared(sender))
+            mt::fake_shared(sender),
+            allocator)
     {
     }
 
@@ -750,7 +782,7 @@ class ObserverPreservingSurfaceFactory : public ms::SurfaceFactory
 {
 public:
     std::shared_ptr<ms::Surface> create_surface(
-        std::shared_ptr<mir::compositor::BufferStream> const&,
+        std::list<ms::StreamInfo> const&,
         mir::scene::SurfaceCreationParameters const& params) override
     {
         using namespace testing;
@@ -788,7 +820,7 @@ struct ApplicationSessionSurfaceOutput : public ApplicationSession
             null_snapshot_strategy,
             stub_session_listener,
             mtd::StubDisplayConfig{},
-            sender)
+            sender, allocator)
     {
     }
 
@@ -865,13 +897,14 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_events_to_surfaces)
 {
     using namespace ::testing;
 
-    MirEvent event;
+    //MirEvent* event;
+    std::unique_ptr<MirEvent> event;
     bool event_received{false};
 
     EXPECT_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillOnce(Invoke([&event, &event_received](auto ev)
+        .WillOnce(Invoke([&event, &event_received](MirEvent const& ev)
                          {
-                             event = ev;
+                             event.reset(ev.clone());
                              event_received = true;
                          }));
 
@@ -883,12 +916,13 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_events_to_surfaces)
     app_session.send_display_config(config);
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({100, 100});
+        .of_size({100, 100})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
     auto surf_id = app_session.create_surface(params, sender);
     auto surface = app_session.surface(surf_id);
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface)
@@ -897,18 +931,19 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface
 
     std::array<TestOutput const*, 2> outputs{{ &high_dpi, &projector }};
 
-    std::array<MirEvent, 2> event;
+    std::array<std::unique_ptr<MirEvent>, 2> event;
     int events_received{0};
 
     ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillByDefault(Invoke([&event, &events_received](auto ev)
+        .WillByDefault(Invoke([&event, &events_received](MirEvent const& ev)
                          {
-                             event[events_received] = ev;
+                             event[events_received].reset(ev.clone());
                              ++events_received;
                          }));
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({100, 100});
+        .of_size({100, 100})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
 
     mf::SurfaceId ids[2];
     std::shared_ptr<ms::Surface> surfaces[2];
@@ -939,8 +974,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface
 
     for (int i = 0; i < 2 ; ++i)
     {
-        EXPECT_THAT(event[i].surface_output.surface_id, Eq(ids[i].as_value()));
-        EXPECT_THAT(&event[i], SurfaceOutputEventFor(*outputs[i]));
+        EXPECT_THAT(event[i]->to_surface_output()->surface_id(), Eq(ids[i].as_value()));
+        EXPECT_THAT(event[i].get(), SurfaceOutputEventFor(*outputs[i]));
     }
 }
 
@@ -950,18 +985,19 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_details_of_the_hightest_scale_fact
 
     std::array<TestOutput const*, 2> outputs{{ &projector, &high_dpi }};
 
-    MirEvent event;
+    std::unique_ptr<MirEvent> event;
     bool event_received{false};
 
     ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillByDefault(Invoke([&event, &event_received](auto ev)
+        .WillByDefault(Invoke([&event, &event_received](MirEvent const& ev)
                               {
-                                  event = ev;
+                                  event.reset(ev.clone());
                                   event_received = true;
                               }));
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({100, 100});
+        .of_size({100, 100})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
 
     auto id = app_session.create_surface(params, sender);
     auto surface = app_session.surface(id);
@@ -984,8 +1020,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_details_of_the_hightest_scale_fact
 
     ASSERT_TRUE(event_received);
 
-    EXPECT_THAT(event.surface_output.surface_id, Eq(id.as_value()));
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
@@ -994,13 +1030,13 @@ TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
 
     std::array<TestOutput const*, 2> outputs{{ &projector, &high_dpi }};
 
-    MirEvent event;
+    std::unique_ptr<MirEvent> event;
     bool event_received{false};
 
     ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillByDefault(Invoke([&event, &event_received](auto ev)
+        .WillByDefault(Invoke([&event, &event_received](MirEvent const& ev)
                               {
-                                  event = ev;
+                                  event.reset(ev.clone());
                                   event_received = true;
                               }));
 
@@ -1018,7 +1054,8 @@ TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
     app_session.send_display_config(config);
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({640, 480});
+        .of_size({640, 480})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
 
     auto id = app_session.create_surface(params, sender);
     auto surface = app_session.surface(id);
@@ -1027,21 +1064,21 @@ TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
     surface->move_to({outputs[0]->width - ((surface->size().width.as_uint32_t()) / 2), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 
     event_received = false;
     // This should be *just* entirely on the projector
     surface->move_to({outputs[0]->width - surface->size().width.as_uint32_t(), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(&event, SurfaceOutputEventFor(projector));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(projector));
 
     event_received = false;
     // This should have a single pixel overlap on the high_dpi
     surface->move_to({outputs[0]->width - (surface->size().width.as_uint32_t() - 1), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
@@ -1050,14 +1087,14 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
 
     std::array<TestOutput const*, 2> outputs {{ &projector, &high_dpi }};
 
-    MirEvent event;
+    std::unique_ptr<MirEvent> event;
     int events_received{0};
 
 
     ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillByDefault(Invoke([&event, &events_received](auto ev)
+        .WillByDefault(Invoke([&event, &events_received](MirEvent const& ev)
                               {
-                                  event = ev;
+                                  event.reset(ev.clone());
                                   events_received++;
                               }));
 
@@ -1075,7 +1112,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     app_session.send_display_config(config);
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({100, 100});
+        .of_size({100, 100})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
 
     auto id = app_session.create_surface(params, sender);
     auto surface = app_session.surface(id);
@@ -1087,8 +1125,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Ge(1));
     auto events_expected = events_received + 1;
 
-    EXPECT_THAT(event.surface_output.surface_id, Eq(id.as_value()));
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 
     // Now solely on the left output
     surface->move_to({0, 0});
@@ -1096,8 +1134,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Eq(events_expected));
     events_expected++;
 
-    EXPECT_THAT(event.surface_output.surface_id, Eq(id.as_value()));
-    EXPECT_THAT(&event, SurfaceOutputEventFor(projector));
+    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(projector));
 
     // Now solely on the right output
     surface->move_to({outputs[0]->width + 100, 100});
@@ -1105,8 +1143,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Eq(events_expected));
     events_expected++;
 
-    EXPECT_THAT(event.surface_output.surface_id, Eq(id.as_value()));
-    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move_only_if_changed)
@@ -1115,13 +1153,11 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move_only_
 
     std::array<TestOutput const*, 2> outputs {{ &projector, &high_dpi }};
 
-    MirEvent event;
     int events_received{0};
 
     ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
-        .WillByDefault(Invoke([&event, &events_received](auto ev)
+        .WillByDefault(Invoke([&events_received](MirEvent const&)
                               {
-                                  event = ev;
                                   events_received++;
                               }));
 
@@ -1139,7 +1175,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move_only_
     app_session.send_display_config(config);
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
-        .of_size({100, 100});
+        .of_size({100, 100})
+        .with_buffer_stream(app_session.create_buffer_stream(properties));
 
     auto id = app_session.create_surface(params, sender);
     auto surface = app_session.surface(id);
